package	jp.co.sds_corp.util.calc;


/**
* 計算式をTokenに分解します。
*/
class Tokenizer {
	public static final int TYPE_INIT = -1;
	public static final int TYPE_EOF = -2;
	public static final int TYPE_NULL = -3;
	public static final int TYPE_NUMBER = 1;
	public static final int TYPE_LITERAL = 2;
	public static final int TYPE_DOT = 8;
	public static final int TYPE_OTHER = 9;
	int	type = TYPE_EOF;
	String exp;
	StringBuffer buf = new StringBuffer();
	int pos = 0;

	Tokenizer( String exp ) {
		this.exp = exp;
	}

	String peekToken() {
		int nPos = pos;
		String ret = nextToken();
		pos = nPos;
		return ret;
	}

	String nextToken() {
		type = TYPE_EOF;
		buf.setLength( 0 );
		if ( pos >= exp.length()) {
			return null;
		}
		for ( ; pos < exp.length(); pos++ ) {
			char c = exp.charAt( pos );
			int nType = getCharType( c );
			if ( nType == TYPE_NULL ) {
				continue;
			}
			if ( type == TYPE_EOF ) {
				type = nType;
			}
			if (( type == TYPE_NUMBER &&
					( nType == TYPE_LITERAL || nType == TYPE_OTHER )) ||
				( type == TYPE_LITERAL &&
					( nType == TYPE_OTHER )))
			{
				break;
			} else {
				buf.append( c );
			}
			if ( type == TYPE_OTHER ) {
				pos++;
				break;
			}
		}
		return buf.toString();
	}

	int getCharType( char c ) {
		if ( Character.isWhitespace( c )) {
			return TYPE_NULL;
		}
		if ( c >= '0' && c <= '9' ) {
			return TYPE_NUMBER;
		}
		if ( Character.isJavaIdentifierStart( c ) ||
			Character.isJavaIdentifierPart( c ))
		{
			return TYPE_LITERAL;
		}
		if ( c == '.' ) {
			return TYPE_DOT;
		}
		return TYPE_OTHER;
	}

	int getType() {
		return type;
	}

	String searchBlanket() throws Exception {
		int depth = 1;
		int start = pos;
		for ( ; pos < exp.length(); pos++ ) {
			char c = exp.charAt( pos );
			if ( c == '(' ) {
				depth++;
			} else
			if ( c == ')' ) {
				depth--;
				if ( depth == 0 ) {
					pos++;
					break;
				}
			}
		}
		if ( depth != 0 ) {
			throw( new SyntaxException( ")が足りません。" ));
		}
		String ret = exp.substring( start, pos-1 );
		return ret;
	}

}
