package	jp.co.sds_corp.util.common;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.List;
import java.util.Map;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
* beanから、値を取り出します。
* a.b.c を指定すると、bean.getA().getB().getC()を行います。
* <p>
* 配列やListオブジェクトに対して、a[0]とすると0番目のオブジェクトを
* 取り出すことができます。
* <p>
* Mapに対しては、a[KEY]とすると、a.get("KEY")を取得できます。
* これらの指定は組み合わせて使用することができます。
* 例えばlist[0].aで、listに含まれている0番目のオブジェクトに対して
* getA()を行います。
*/
public class BeanParser
{
	static Pattern pattern = Pattern.compile( "(.*)\\[([\\w.]+)\\]" );
	Object bean;

	/**
	* コンストラクタ
	* @param	bean	値を取り出すbean
	*/
	public BeanParser( Object bean ) {
		this.bean = bean;
	}

	private Object get( String key )
		throws NoSuchMethodException,
				IllegalAccessException,InvocationTargetException
	{
		String idx = null;
		boolean bIdx = false;
		Matcher matcher = pattern.matcher( key );
		if ( matcher.find()) {
			key = matcher.group( 1 );
			idx = matcher.group( 2 );
			bIdx = true;
		}
		Object obj = null;
		String func = "get" + Character.toUpperCase( key.charAt( 0 ))
						+ key.substring( 1 );
						// + key.substring( 1 ).toLowerCase();
		Method mtd = bean.getClass().getMethod( func );
		obj = mtd.invoke( bean );
		if ( !bIdx ) {
			return obj;
		}
		if ( obj.getClass().isArray()) {
			Object[] arr = (Object[] )obj;
			int nIdx = Integer.valueOf( idx ).intValue();
			obj = arr[nIdx];
		}
		if ( obj instanceof List ) {
			List<Object> list = (List<Object> )obj;
			int nIdx = Integer.valueOf( idx ).intValue();
			obj = list.get( nIdx );
		}
		if ( obj instanceof Map ) {
			Map<String,Object> map = (Map<String,Object> )obj;
			obj = map.get( idx );
		}
		return obj;
	}

	/**
	* beanから値を取り出します。
	* @param	pkey	メンバを指定する文字列
	* @return	取り出した値
	*/
	public Object getObject( String pkey )
		throws NoSuchMethodException,
				IllegalAccessException,InvocationTargetException
	{
		BeanParser parser = this;
		Object tgt = bean;
		String key = pkey;
		String name;
		for ( ;; ) {
			int idx = key.indexOf( "." );
			if ( idx < 0 ) {
				break;
			}
			name = key.substring( 0, idx );
			key = key.substring( idx + 1 );
			tgt = parser.get( name );
			parser = new BeanParser( tgt );
		}
		Object ret = parser.get( key );
		return ret;
	}

	/**
	* beanのgetterと合致するsetterを呼び出します。
	* @param	dst		copy先のオブジェクト
	*/
	public void copyTo( Object dst ) {
		Method[] setter = dst.getClass().getMethods();
		for ( int i = 0; i < setter.length; i++ ) {
			if ( !setter[i].getName().startsWith( "set" )) {
				continue;
			}
			Class<?>[] setPrmType = setter[i].getParameterTypes();
			if ( setPrmType.length != 1 ) {
				continue;
			}
			String name = "g" + setter[i].getName().substring( 1 );
			try {
				Method getter = bean.getClass().getMethod( name );
				Object ret = getter.invoke( bean );
				Object[] setPrm = new Object[1];
				setPrm[0] = ret;
				setter[i].invoke( dst, setPrm );
			} catch ( Exception ex ) {
				continue;
			}
		}
	}

}
