package jp.co.sds_corp.util.common;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.PrintStream;
import java.net.URL;

/**
* 変数のダンプ出力クラス。
*/
public class DumpTool
{
	boolean	showAddress = true;	// アドレス表示フラグ
	boolean	showCharacter = true;	// 文字表示フラグ

	/**
	* 入力streamから読み込んだbyte配列を16進ダンプし、出力streamに書き込む。
	* @param	is		入力stream
	* @param	os		出力stream
	*/
	public void encode( InputStream is, OutputStream os ) throws Exception {
		PrintStream ps = new PrintStream( os );
		int addr = 0;
		byte[] buf = new byte[16];
		for ( ;; ) {
			int len = is.read( buf );
			if ( len < 0 ) {
				break;
			}
			if ( showAddress ) {
				ps.printf( "%04X: ", addr );
			}
			for ( int i = 0; i < len; i++ ) {
				int ib = buf[i] & 0xff;
				ps.printf( "%02X ", ib );
				if ( i == 7 ) {
					ps.print( "  " );
				}
			}
			if ( showCharacter ) {
				for ( int i = len; i < 16; i++ ) {
					ps.print( "   " );
					if ( i == 7 ) {
						ps.print( "  " );
					}
				}
				ps.print( " " );
				byte[] cb = new byte[1];
				for ( int i = 0; i < len; i++ ) {
					cb[0] = buf[i];
					String cs = new String( cb );
					char ch = cs.charAt( 0 );
					int type = Character.getType( ch );
					if ( type == Character.CONTROL ||
						type == Character.OTHER_SYMBOL )
					{
						ps.print( "." );
					} else {
						ps.print( cs );
					}
				}
			}
			ps.println();
			addr += len;
		}
	}

	/**
	* byte配列の指定された位置のデータを16進ダンプした文字列を返します。
	* @param	buf		ダンプする文字列
	* @param	offset	ダンプ開始位置
	* @param	count	ダンプするbyte数
	* @return	16進ダンプした文字列
	*/
	public String dump( byte[] buf, int offset, int count ) throws Exception {
		ByteArrayInputStream is =
			new ByteArrayInputStream( buf, offset, count );
		ByteArrayOutputStream os = new ByteArrayOutputStream();
		encode( is, os );
		String ret = os.toString();
		return ret;
	}

	/**
	* byte配列を16進ダンプした文字列を返します。
	* @param	buf		ダンプする文字列
	* @return	16進ダンプした文字列
	*/
	public String dump( byte[] buf ) throws Exception {
		String ret = dump( buf, 0, buf.length );
		return ret;
	}

	/**
	* Streamを読み込み、16進ダンプした文字列を返します。
	* @param	is		読み込むStream
	* @return	16進ダンプした文字列
	*/
	public String dump( InputStream is ) throws Exception {
		ByteArrayOutputStream os = new ByteArrayOutputStream();
		encode( is, os );
		is.close();
		String ret = os.toString();
		return ret;
	}

	/**
	* FileからStreamを読み込み、16進ダンプした文字列を返します。
	* @param	file	読み込むファイル
	* @return	16進ダンプした文字列
	*/
	public String dump( File file ) throws Exception {
		FileInputStream is = new FileInputStream( file );
		String ret = dump( is );
		return ret;
	}

	/**
	* URLからStreamを読み込み、16進ダンプした文字列を返します。
	* @param	url		読み込むURL
	* @return	16進ダンプした文字列
	*/
	public String dump( URL url ) throws Exception {
		InputStream is = url.openStream();
		String ret = dump( is );
		return ret;
	}

	/**
	* アドレス表示フラグを設定します。
	* @param	var		設定するデータ
	*/
	public void setShowAddress( boolean var ) {
		this.showAddress = var;
	}
	/**
	* アドレス表示フラグを取得します。
	* @return	アドレス表示フラグ
	*/
	public boolean isShowAddress() {
		return showAddress;
	}

	/**
	* 文字表示フラグを設定します。
	* @param	var		設定するデータ
	*/
	public void setShowCharacter( boolean var ) {
		this.showCharacter = var;
	}
	/**
	* 文字表示フラグを取得します。
	* @return	文字表示フラグ
	*/
	public boolean isShowCharacter() {
		return showCharacter;
	}

	public static void main( String[] args ) {
		try {
			DumpTool dump = new DumpTool();
			for ( String file : args ) {
				String str = dump.dump( new File( file ));
				System.out.println( str );
			}
		} catch ( Exception ex ) {
			ex.printStackTrace();
		}
	}

}
