package	jp.co.sds_corp.util.common;

import java.io.DataInputStream;
import java.net.URL;
import java.net.URLClassLoader;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.jar.JarEntry;
import java.util.jar.JarFile;

/**
* JarファイルからClassファイルを読み出すClassLoaderです。
* @author	小見山　聡
*/
public class JarClassLoader extends ClassLoader
{
	JarFile		jf;
	Hashtable<String,Object>	classDef = new Hashtable<String,Object>();
	URLClassLoader	loader;

	/**
	* 指定されたJarファイルで初期化します。
	* @param	jar		読み込むJarファイル
	*/
	public JarClassLoader( String jar, ClassLoader parent ) throws Exception {
		super( parent );
		try {
			URL[] urls = { new URL( "jar:file:" + jar + "!/" ) };
			loader = new URLClassLoader( urls, parent );
			jf = new JarFile( jar );
			loadAllClasses();
		} catch ( Exception ex ) {
			throw( ex );
		}
	}

	/**
	* 指定されたJarファイルで初期化します。
	* @param	jar		読み込むJarファイル
	*/
	public JarClassLoader( String jar ) throws Exception {
		super();
		try {
			jf = new JarFile( jar );
			loadAllClasses();
		} catch ( Exception ex ) {
			throw( ex );
		}
	}

	/**
	* Jarに含まれる全てのクラスを読み込みます。
	*/
	protected void loadAllClasses() {
		if ( jf == null ) {
			return;
		}
		Enumeration<JarEntry> en = jf.entries();
		while ( en.hasMoreElements()) {
			JarEntry je = (JarEntry )en.nextElement();
			if ( je.isDirectory()) {
				continue;
			}
			String className = je.getName().replace( '/', '.' );
			if ( className.endsWith( ".class" )) {
				className = className.substring( 0, className.length() - 6 );
			}
			byte[] b = loadClassData( className );
			if ( b != null ) {
				classDef.put( className, b );
			}
		}
	}

	/**
	* 指定されたリソースを探します。
	* @param	name	リソース名
	* @return	リソースを読み込むための URL。
	*/
	public URL findResource( String name ) {
		return loader.findResource( name );
	}

	/**
	* 指定されたクラスを探します。
	* @param	className	クラス名
	* @return	結果の Class オブジェクト
	*/
	public Class<?> findClass( String className ) {
		SecurityManager sm = System.getSecurityManager();
		Object obj = classDef.get( className );
		if ( obj == null ) {
			return null;
		}
		if ( obj instanceof Class ) {
			return (Class<?> )obj;
		} else if ( obj instanceof byte[] ) {
			byte[] b = (byte[] )obj;
			if ( sm != null ) {
				int i = className.lastIndexOf( "." );
				if ( i >= 0 ) {
					System.getSecurityManager().checkPackageAccess( className.substring( 0, i ));
				}
			}
			Class<?> cls = defineClass( className, b, 0, b.length );
			classDef.put( className, cls );
			return cls;
		}
		return null;
	}

	/**
	* 指定されたクラスを探します。
	* @param	className	クラス名
	* @return	読み込んだ Class オブジェクト
	*/
	private byte[] loadClassData( String className ) {
		try {
			String drvpath = className.replace( '.', '/' );
			JarEntry je = jf.getJarEntry( drvpath + ".class" );
			byte[] buf = new byte[(int )je.getSize()];
			DataInputStream di = new DataInputStream( jf.getInputStream( je ) );
			di.readFully( buf );
			di.close();
			return buf;
		} catch ( Exception ex ) {
		}
		return null;
	}
}

