package	jp.co.sds_corp.util.common;

import java.io.PrintStream;
import java.io.PrintWriter;

/**
 * 別のExceptionをラップします。
 * <p>
 * メソッド内で発生したさまざまな例外について、同じ扱いにして
 * そのメソッドからは一つのthrowsにしたいといった場合に使います。<br>
 * あるいは、catchした例外に対してメッセージを付加してrethrowする場合も
 * 有効です。printStackTrace()は元の例外のものになります。
 * </p>
 */
public class NestedException extends Exception
{
	private static final long serialVersionUID = 1L;
	private Throwable original = null;

	/**
	* 詳細メッセージを持たないNestedExceptionを構築します。
	*/
	public NestedException() {
		super();
	}

	/**
	* 詳細メッセージを持つNestedExceptionを構築します。
	* @param	message		詳細メッセージ
	*/
	public NestedException( String message ) {
		super( message );
	}

	/**
	* 詳細メッセージを持つNestedExceptionを構築します。
	* @param	message		詳細メッセージ
	* @param	ex			元のException
	*/
	public NestedException( String message, Throwable ex ) {
		super( "Exception [" + ex.getClass() + "] occured.\n" +
				"Message:[" + ex.getMessage() + "]\n" + message );
		original = ex;
	}

	/**
	* 元のExceptionを返します。
	* @return	最初に発生した例外
	*/
	public Throwable getOriginal() {
		return original;
	}

	/**
	* スタックとレースを標準エラー出力に出力します。
	*/
	public void printStackTrace() {
		super.printStackTrace();
		if ( original != null ) {
			System.err.println( "Caused by:" + original );
			original.printStackTrace();
		}
	}
	/**
	* スタックとレースを指定されたStreamに出力します。
	* @param	o		出力するStream
	*/
	public void printStackTrace( PrintStream o ) {
		if ( original != null ) {
			o.println( "Caused by:" + original );
			original.printStackTrace( o );
		}
		super.printStackTrace( o );
	}
	/**
	* スタックとレースを指定されたPrintWriterに出力します。
	* @param	o		出力するPrintWriter
	*/
	public void printStackTrace( PrintWriter o ) {
		super.printStackTrace( o );
		if ( original != null ) {
			o.println( "Caused by:" + original );
			original.printStackTrace( o );
		}
	}
}
