package	jp.co.sds_corp.util.config;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.InputStream;
import java.net.URL;
import java.util.Iterator;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import jp.co.sds_corp.util.log.ConsoleLogger;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.w3c.dom.Text;

/**
* {@link IConfiguration}の、DOMの実装を提供します。
*/
public class DOMConfiguration extends AbstractConfiguration
{
	private static final long serialVersionUID = 1L;
	static ConsoleLogger logger = ConsoleLogger.getInstance();
	protected File				file;
    protected ConfigurationDocument			document;
    protected DOMConfiguration	parent;
    protected Element			current;

    public DOMConfiguration( String path, String charset ) throws Exception {
        this( new File( path ), charset );
    }

    public DOMConfiguration( String path ) throws Exception {
        this( new File( path ) );
    }

    public DOMConfiguration( File file ) throws Exception {
		if ( !file.exists()) {
			throw( new FileNotFoundException( file.toString() ));
		}
        this.file = file;
		FileInputStream str = new FileInputStream( file );
		parse( str );
		str.close();
    }

    public DOMConfiguration( File file, String charset ) throws Exception {
		this( file );
    }

    public DOMConfiguration( URL url ) throws Exception {
		if ( url == null ) {
			throw( new FileNotFoundException());
		}
		parse( url.openStream() );
    }

	public static DOMConfiguration findResource( String file ) throws Exception {
		URL config = DOMConfiguration.class.getClassLoader().getResource( file );
	    if ( config == null ) {
			throw( new FileNotFoundException( file ));
		}
		DOMConfiguration ret = new DOMConfiguration( config );
		return ret;
	}

	public void parse( InputStream str ) throws Exception {
        this.parent = null;
		DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
		DocumentBuilder builder = factory.newDocumentBuilder();
		Document doc = builder.parse( str );
		document = new ConfigurationDocument( doc );
        current = document.getDocumentElement();
		readProperty(this);
	}

	void readProperty(IConfiguration config) throws Exception {
		Iterator<IConfiguration> itr = config.getChilds( "property" );
		while ( itr.hasNext() ) {
			IConfiguration prop = itr.next();
			String name = prop.getAttributes().get( "name", null );
			String value = prop.getAttributes().get( "value", null );
			if ( name == null || value == null ) {
				continue;
			}
			document.setProperty( name, value );
		}
	}

	public static DOMConfiguration createDocument( String name )
		throws ParserConfigurationException
	{
		DOMConfiguration root = new DOMConfiguration();
		DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
		DocumentBuilder builder = factory.newDocumentBuilder();
		root.document = new ConfigurationDocument(builder.newDocument());
		Element ele = root.document.createElement( name );
		root.document.appendChild( ele );
		root.parent = null;
		root.current = root.document.getDocumentElement();
		return root;
	}

	private DOMConfiguration() throws ParserConfigurationException {
	}

	private DOMConfiguration( DOMConfiguration parent, Element current ) {
		this.parent = parent;
		this.document = parent.document;
		this.current = current;
		file = parent.file;
		String subFile = current.getAttribute( "subConfig" );
		if ( subFile != null && subFile.length() != 0 ) {
			try {
				file = new File( file.getParent(), subFile );
				logger.debug( this, "sub configuration file is opening :" + file );
				DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
				DocumentBuilder builder = factory.newDocumentBuilder();
				Document doc = builder.parse( file );
				this.current = doc.getDocumentElement();
			} catch ( Exception ex ) {
				logger.error( this, "file not found", ex );
			}
		}
	}

	public IConfiguration getParent() {
		if ( parent == null ) {
			throw new RuntimeException( "current is toplevel." );
		}
		return parent;
	}

	// return first detected node
	public IConfiguration getChild( String name ) {
		Element child = getElement( current, name );
		if ( child == null ) {
//			throw new UnspecifiedPropertyException(
//				"ノード<" + current.getNodeName() + ">の中の"
//				+ "子 (" + name + ") " );
			return null;
		}
		return new DOMConfiguration( this, child );
	}
	public Iterator<IConfiguration> getChilds() {
		return new Iterator<IConfiguration>() {
			private Node c;
			{
				c = current.getFirstChild();
				while ( c != null && c.getNodeType() != Node.ELEMENT_NODE ) {
					c = c.getNextSibling();
				}
			}
			public boolean hasNext() {
				return c != null;
			}
			public IConfiguration next() {
				Element result = (Element )c;
				do {
					c = c.getNextSibling();
				} while ( c != null && c.getNodeType() != Node.ELEMENT_NODE );
				return new DOMConfiguration( DOMConfiguration.this, result );
			}
			public void remove() {
				throw new UnsupportedOperationException();
			}
		};
	}
	public Iterator<IConfiguration> getChilds( final String name ) {
		return new Iterator<IConfiguration>() {
			private Element c = getElement( current, name );
			public boolean hasNext() {
				return c != null;
			}
			public IConfiguration next() {
				Element result = c;
				c = getNextSibling( c );
				return new DOMConfiguration( DOMConfiguration.this, result );
			}
			public void remove() {
				throw new UnsupportedOperationException();
			}
		};
	}

	private IConfiguration       attributes = null;
	public IConfiguration getAttributes() {
		if ( attributes == null ) {
			attributes = new AttributeConfiguration();
		}
		return attributes;
	}

	public String getName() {
		return current.getNodeName();
	}

	/** */
	public String get() throws UnspecifiedPropertyException {
		String value = getContentString( current );
		if ( value == null ) {
			throw new UnspecifiedPropertyException(
				current.getNodeName() + " node" );
		}
		return value;
	}
	public String get( String key ) throws UnspecifiedPropertyException {
		String value = null;
		Element child = getElement( current, key );
		if ( child != null ) {
			value = getContentString( child );
		}
		if ( value == null ) {
			throw new UnspecifiedPropertyException(
				"ノード<" + current.getNodeName() + ">の中の"
				+ "キー (" + key + ") " );
		}
		return value;
	}

	protected static Element getElement( Node node, String name ) {
		if ( node == null ) {
			return null;
		}
		NodeList list = node.getChildNodes();

		for ( int i = 0; i < list.getLength(); i++ ) {
			if ( list.item( i ).getNodeName().equals( name )) {
				return (Element )list.item( i );
			}
		}
		return null;
	}
	public static Element getNextSibling( Node element ) {
		Node node = element.getNextSibling();
		while ( node != null
				&& !element.getNodeName().equals( node.getNodeName())) {
			node = node.getNextSibling();
		}
		return (Element )node;
	}
	protected String getContentString( Node node ) {
		if ( node == null ) {
			return null;
		}
		Node current = node.getFirstChild();
		if ( current == null ) {
			return null;
		}
		StringBuffer buf = new StringBuffer();
		for ( ; current != null; current = current.getNextSibling() ) {
			switch ( current.getNodeType() ) {
			case Node.TEXT_NODE:
				buf.append( current.getNodeValue() );
				break;
			case Node.ENTITY_REFERENCE_NODE:
				buf.append( '&' ).append( current.getNodeName()).append( ';' );
				break;
			case Node.ELEMENT_NODE:
			case Node.COMMENT_NODE:
			default:
			}
		}
		String ret = buf.toString();
		ret = document.replaceValue( ret );
		return ret;
	}

	protected IConfiguration addChild( DOMConfiguration config, Element org ) {
		Element ele = document.createElement( org.getNodeName() );
		config.current.appendChild( ele );
		DOMConfiguration child = new DOMConfiguration( config, ele );

		NamedNodeMap map = org.getAttributes();
		for ( int i = 0; i < map.getLength(); i++ ) {
			Node item = map.item( i );
			ele.setAttribute( item.getNodeName(), item.getNodeValue() );
		}

		NodeList list = org.getChildNodes();
		for ( int i = 0; i < list.getLength(); i++ ) {
			Node item = list.item( i );
			switch ( item.getNodeType() ) {
			case Node.TEXT_NODE:
				Text txt = document.createTextNode( item.getNodeValue() );
				ele.appendChild( txt );
				break;
			case Node.ELEMENT_NODE:
				addChild( child, (Element )item );
				break;
			default:
			}
		}
		return child;
	}
	public IConfiguration addChild( IConfiguration config ) {
		DOMConfiguration child = (DOMConfiguration )addChild( this,
									((DOMConfiguration )config).current );
		return child;
	}
	public IConfiguration addChild( String key ) {
		Element ele = document.createElement( key );
		current.appendChild( ele );
		DOMConfiguration child = new DOMConfiguration( this, ele );
		return child;
	}
	public IConfiguration addChild( String key, String value ) {
		DOMConfiguration child = (DOMConfiguration )addChild( key );
		child.set( value );
		return child;
	}

	public void set( String value ) {
		Node c = current.getFirstChild();
		while ( c != null && c.getNodeType() != Node.TEXT_NODE ) {
			c = c.getNextSibling();
		}
		Text txt;
		if ( c == null ) {
			txt = document.createTextNode( value );
			current.appendChild( txt );
		} else {
			c.setNodeValue( value );
		}
	}

	public void setAttribute( String key, String value ) {
		current.setAttribute( key, value );
	}

	public class AttributeConfiguration extends AbstractConfiguration {
		public IConfiguration getParent() {
			return DOMConfiguration.this;
		}
		/** */
		public String get() {
			throw new UnsupportedOperationException();
		}
		public String get( String key ) throws UnspecifiedPropertyException {
			String value = current.getAttribute( key );
			if ( value == null || value.length() == 0 ) {
				throw new UnspecifiedPropertyException(
					"ノード<" + current.getNodeName() + ">の中の"
					+ "属性 (" + key + ") " );
			}
			return value;
		}
	}

}
