package	jp.co.sds_corp.util.config;

import java.io.OutputStream;
import java.io.OutputStreamWriter;
import java.io.Writer;
import java.net.URL;

import javax.xml.transform.OutputKeys;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;
import javax.xml.transform.stream.StreamSource;

/**
* {@link IConfiguration}の内容をXMLに出力します。
* 現在、{@link DOMConfiguration}を前提とした実装となっています。
*/
public class DOMWriter
{
	private Writer out;

	/**
	* 出力先を指定してDOMWriterを構築します。
	* @param	out		出力先
	*/
	public DOMWriter( Writer out ) {
		this.out = out;
	}

	/**
	* 出力先を指定してDOMWriterを構築します。
	* @param	out		出力先
	*/
	public DOMWriter( OutputStream out ) {
		this.out = new OutputStreamWriter( out );
	}

	/**
	* 指定された設定情報を出力します。
	* @param	config	設定情報
	* @param	xsltUrl	出力を整形するXSLTファイルのURL。
	*					nullを指定すると、デフォルトのstylesheetを使用する。
	*/
	public void write( IConfiguration config, String name, URL xsltUrl )
		throws Exception
	{
		DOMConfiguration dom = (DOMConfiguration )config;
		TransformerFactory factory = TransformerFactory.newInstance();
		if ( xsltUrl == null ) {
			xsltUrl = getClass().getResource( "/resource/style.xsl" );
		}
		StreamSource xslt = new StreamSource( xsltUrl.toString() );
		Transformer trans = factory.newTransformer( xslt );

		DOMSource source = new DOMSource( dom.current, name );
		StreamResult result = new StreamResult( out );
		trans.transform( source, result );
	}
	/**
	* 指定された設定情報を出力します。
	* @param	config	設定情報
	* @param	indent	出力を整形するか
	*/
	public void write( IConfiguration config, String name, boolean indent )
		throws Exception
	{
		DOMConfiguration dom = (DOMConfiguration )config;
		TransformerFactory factory = TransformerFactory.newInstance();
		Transformer trans = factory.newTransformer();
		trans.setOutputProperty( OutputKeys.INDENT, "yes" );

		DOMSource source = new DOMSource( dom.current, name );
		StreamResult result = new StreamResult( out );
		trans.transform( source, result );
	}
	/**
	* 指定された設定情報を出力します。
	* @param	config	設定情報
	*/
	public void write( IConfiguration config, String name ) throws Exception {
		write( config, name, true );
	}
}
