package	jp.co.sds_corp.util.file;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.net.URI;

/**
* {@link File}の拡張機能です。
* 現在のところ{@link #copyTo}のみ実装されています。
*/
public class CmnFile extends File
{
	private static final long serialVersionUID = 1L;

	/**
	* 指定されたパス名文字列を抽象パス名に変換して、
	* 新しい File のインスタンスを生成します。
	* 指定された文字列が空の文字列の場合、結果は空の抽象パス名になります。
	* @param	file	パス名文字列
	*/
	public CmnFile( String file ) {
		super( file );
	}

	/**
	* 抽象パス名および子パス名文字列から新しい File のインスタンスを生成します。
	* @param	parent	親抽象パス名
	* @param	child	子パス名文字列
	*/
	public CmnFile( File parent, String child ) {
		super( parent, child );
	}

	/**
	* 親パス名文字列および子パス名文字列から
	* 新しい File のインスタンスを生成します。
	* @param	parent	親パス名文字列
	* @param	child	子パス名文字列
	*/
	public CmnFile( String parent, String child ) {
		super( parent, child );
	}

	/**
	*
	* 指定された file: URI を抽象パス名に変換して、
	* 新しい File のインスタンスを生成します。
	* @param	uri		階層型の絶対 URI。
	*		 			形式は、"file"、パス、権限、クエリー、フラグメント。
	*					パスは必ず指定する。
	*					権限、クエリー、およびフラグメントは定義しない
	*/
	public CmnFile( URI uri ) {
		super( uri );
	}

	/**
	* この抽象パス名が示すファイルを指定されたパスにコピーします。
	* @param	dest	コピー先
	*/
	public void copyTo( File dest ) throws Exception {
		FileWriter out = new FileWriter( dest );
		BufferedReader rd = new BufferedReader( new FileReader( this ));
		char[] buf = new char[10240];
		int nRet;
		for ( ;; ) {
			nRet = rd.read( buf );
			if ( nRet <= 0 ) {
				break;
			}
			out.write( buf, 0, nRet );
		}
		rd.close();
		out.flush();
		out.close();
	}
}
