package	jp.co.sds_corp.util.math;


/**
* Gauss−Seidelの反復法
*/
public class GaussSeidel
{
	public static int MAX_ITER = 10000;
	public static double EPS   = 1E-6;

	public static void setEps(int eps) {
		EPS = eps;
	}
	/**
	* 最大反復回数を設定する。
	*/
	public static void setMax_iter(int max_iter) {
		MAX_ITER = max_iter;
	}

	/**
	* Gauss−Seidelの反復法によって次数の低い連立方程式を解く。
	* @param	a		行列A
	* @param	x		初期近似解および最終解
	* @param	b		列ベクトル
	*/
	public static boolean doubleSolve(double[][] a, double[] x, double[] b) {
		int n = b.length;
		for (int iter=1; iter<MAX_ITER; iter++) {
			boolean ok = true;
			for (int i=0; i<n; i++) {
				double s = b[i];

				for (int j=0; j<i; j++) {
					s -= a[i][j] * x[j];
				}
				for (int j=i+1; j<n; j++) {
					s -= a[i][j] * x[j];
				}
				s /= a[i][i];

				if (ok && Math.abs(x[i] - s) > EPS * (1 + Math.abs(s))) {
					ok = false; 
				}
				x[i] += 1.0 * (s - x[i]);
				// SOR法の場合は1.0をちょっと大きくすると
				// 収束が早くなるが誤差を含みます。
			}
			if (ok) {
				System.out.println("繰り返し回数:" + iter);
				return ok;
			}
		}
		return false;
	}
}
