package	jp.co.sds_corp.util.net;

import java.net.InetAddress;
import java.net.SocketException;
import java.net.SocketTimeoutException;
import java.net.UnknownHostException;
import java.util.Random;
import java.util.Vector;

import jp.co.sds_corp.util.common.CmnDate;

/**
* pingコマンドを実装するクラスです。
*/
public class Ping
{
	Result[]	res;
	InetAddress local;
	InetAddress	addr;
	ICMPSocket	sockSend;
	ICMPSocket	sockRecv;
	int			ident;
	int			ttl = 64;
	Vector<IPingListener>	vecLsn = new Vector<IPingListener>();

	/**
	* 指定されたアドレスに対してオブジェクトを構築します。
	* @param	addr	ping先
	*/
	public Ping( InetAddress addr ) throws SocketException, UnknownHostException
	{
		this.addr = addr;
		local = InetAddress.getLocalHost();
		sockRecv = new ICMPSocket( local );
		// sockSend = new ICMPSocket();
		sockSend = sockRecv;

		ident = new Random().nextInt( 128 );
	}

	/**
	* 指定されたアドレスに対するpingを実行します。
	* @return	指定されたアドレスから応答があればtrue,なければfalse
	*/
	public boolean check() throws Exception {
		check( 1 );
		if ( res[0].isAlive()) {
			return true;
		}
		return false;
	}

	/**
	* 指定されたアドレスに対するpingを指定回数実行します。
	* @param	times	pingを実行する回数
	*/
	public void check( int times ) throws Exception {
		res = new Result[times];
		for ( int i = 0; i < times; i++ ) {
			res[i] = new Result();
			if ( i > 0 && res[i-1].isAlive() ) {
				try {
//					Thread.sleep( 1000 );
					Thread.sleep( 100 );
				} catch ( Exception ex ) {
				}
			}
			try {
				res[i].alive = false;
				sendEcho( i );
				res[i].alive = true;
			} catch ( SocketException ex ) {
				fireEvent( res[i] );
				throw( ex );
			} catch ( SocketTimeoutException ex ) {
				fireEvent( res[i] );
				throw( ex );
			}
			fireEvent( res[i] );
		}
	}

	private void sendEcho( int i ) throws Exception {
		byte[] sbuf = "01234567890123456789012345678901".getBytes();
		ICMPPacket send = new ICMPPacket( sbuf, sbuf.length, addr,
									ICMPPacket.Type.ECHO_REQUEST );
		byte[] rbuf = new byte[1024];
		ICMPPacket recv = new ICMPPacket( rbuf, rbuf.length, addr,
									ICMPPacket.Type.ECHO_REPLY );
		send.setSequence( i );
		res[i].send = send;
		send.setIdentifier( ident );

		sockSend.setTTL( ttl );
		CmnDate start = new CmnDate();
		sockSend.send( send );

		for ( ;; ) {
			recv.setLength( rbuf.length );
			sockRecv.receive( recv );
			if ( recv.getType() == ICMPPacket.Type.TIME_EXCEEDED ) {
				break;
			}
			if ( recv.getType() == ICMPPacket.Type.ECHO_REPLY &&
				recv.getIdentifier() == ident )
			{
				break;
			}
		}
		CmnDate now = new CmnDate();

		res[i].recv = recv;
		res[i].response = now.getTimeInMillis() - start.getTimeInMillis();
	}

	/**
	* pingを終了します。
	*/
	public void close() {
		sockSend.close();
		sockRecv.close();
	}

	/**
	* 全ての実行結果を返します。
	*/
	public Result[] getResults() {
		return res;
	}

	/**
	* 指定された回数目の結果を返します。
	*/
	public Result getResult( int i ) {
		return res[i];
	}

	/**
	* TTL(ttl)を設定します。
	* @param	var		設定するデータ
	*/
	public void setTTL( int var ) {
		this.ttl = var;
	}
	/**
	* TTL(ttl)を取得します。
	* @return	ttlの内容
	*/
	public int getTTL() {
		return ttl;
	}

	/**
	* 受信を監視する{@link IPingListener}を追加します。
	* @param	l		追加するIPingListener
	*/
	public void addPingListener( IPingListener l ) {
		vecLsn.add( l );
	}
	/**
	* 受信を監視する{@link IPingListener}を削除します。
	* @param	l		削除するIPingListener
	*/
	public void removePingListener( IPingListener l ) {
		vecLsn.remove( l );
	}

	private void fireEvent( Result res ) {
		for ( int i = 0; i < vecLsn.size(); i++ ) {
			IPingListener l = (IPingListener )vecLsn.get( i );
			l.pingReceived( res );
		}
	}

	/**
	* Pingの結果を保持します。
	*/
	public static class Result
	{
		boolean		alive;
		long		response;
		ICMPPacket	send;
		ICMPPacket	recv;

		/**
		* 一定時間内に応答があった場合にtrueを返します。
		*/
		public boolean isAlive() {
			return alive;
		}
		/**
		* 応答までの時間をミリ秒で返します。
		*/
		public long getResponse() {
			return response;
		}
		/**
		* 送信したパケットを返します。
		*/
		public ICMPPacket getSendPacket() {
			return send;
		}
		/**
		* 受信したパケットを返します。
		*/
		public ICMPPacket getReceivePacket() {
			return recv;
		}
	}
}
