package	jp.co.sds_corp.util.script;

import jp.co.sds_corp.util.common.CmnProperty;
import jp.co.sds_corp.util.config.IConfigSavable;
import jp.co.sds_corp.util.config.IConfigurable;

/**
* Scriptを実行するクラスの定義です。
*/
public interface IScriptParser extends IConfigurable, IConfigSavable
{
	/**
	* スクリプトを設定します。
	* @param	var		設定するデータ
	*/
	public void setScript( String var );
	/**
	* スクリプトを取得します。
	* @return	スクリプト
	*/
	public String getScript();
	/**
	* スクリプト言語の種類を返します。
	* @return	スクリプト言語の種類
	*/
	public String getLanguage();
	/**
	* 初期情報の変換に使用するリソースを設定します。
	* @param	prop	リソース
	*/
	public void setProperty( CmnProperty prop );
	/**
	* 初期情報の変換に使用するリソースを返します。
	* @return	リソース
	*/
	public CmnProperty getProperty();
	/**
	* スクリプトの文字コードを設定します。
	* @param	var		文字コード
	*/
	public void setCharset( String var );
	/**
	* スクリプトの文字コードを取得します。
	* @return	文字コード
	*/
	public String getCharset();

	/**
	* スクリプトに変数を設定します。
	* スクリプト内からは、名前でオブジェクトを参照します。
	* @param	name	変数名
	* @param	obj		オブジェクト
	*/
	public void setAttribute( String name, Object obj ) throws Exception;
	/**
	* スクリプトに設定されている変数を取得します。
	* @param	name	変数名
	* @return	オブジェクト
	*/
	public Object getAttribute( String name ) throws Exception;

	/**
	* スクリプトを実行します。
	* @return	実行結果
	*/
	public String execute() throws Exception;
	/**
	* スクリプトの終了処理を行います。
	*/
	public void exit();
}
