package	jp.co.sds_corp.util.thread;

/**
Threadの状態をモニタするユーティリティです。
*/
public class ThreadMonitor
{
	ThreadGroup root;

	/**
	* オブジェクトを初期化します。
	*/
	public ThreadMonitor() {
		this( null );
	}

	/**
	* 指定されたThreadGroupの状態をモニタするようにオブジェクトを作成します。
	* @param	root	モニタするトップレベルのThreadGroup
	*/
	public ThreadMonitor( ThreadGroup root ) {
		if ( root == null ) {
			root = getTopLevelThreadGroup();
		}
		this.root = root;
	}

	/**
	* トップレベルのThreadGroupを取得します。
	* @return	トップレベルのThreadGroup
	*/
	public static ThreadGroup getTopLevelThreadGroup() {
		ThreadGroup top, parent;
		top = Thread.currentThread().getThreadGroup();
		for ( ;; ) {
			if (( parent = top.getParent()) == null ) {
				break;
			}
			top = parent;
		}
		return top;
	}

	/**
	* Threadの一覧を文字列で取得します。
	* @return	Threadの一覧
	*/
	public String getString() {
		StringBuffer buf = new StringBuffer();
		writeGroup( buf, root, 0 );
		return buf.toString();
	}

	private void writeGroup( StringBuffer buf, ThreadGroup group, int indent ) {
		indent( buf, indent );
		buf.append( '\"' ).append( group.getName()).append( '\"' );
		buf.append( " (max:" ).append( group.getMaxPriority());
		if ( group.isDaemon()) {
			buf.append( ",daemon" );
		}
		buf.append( ") {\n" );

		int num = group.activeGroupCount();
		if ( num > 0 ) {
			ThreadGroup[] list = new ThreadGroup[num];
			num = group.enumerate( list, false );
			for ( int i = 0; i < num; i++ ) {
				writeGroup( buf, list[i], indent + 1 );
			}
		}
		num = group.activeCount();
		if ( num > 0 ) {
			Thread[] list = new Thread[num];
			num = group.enumerate( list, false );
			for ( int i = 0; i < num; i++ ) {
				Thread thread = list[i];
				indent( buf, indent + 1 );
				buf.append( '\"' ).append( thread.getName()).append( '\"' );
				buf.append( " (pri:" ).append( thread.getPriority());
				if ( thread.isDaemon()) {
					buf.append( ",daemon" );
				}
				buf.append( ")\n" );
			}
		}

		indent( buf, indent );
		buf.append( "}\n" );
	}

	private void indent( StringBuffer buf, int indent ) {
		for( int i = 0; i < indent; i++ ) {
			buf.append( "    " );
		}
	}
}
