package jp.co.sds_corp.util.crypt;

import java.io.File;
import java.io.FileInputStream;
import java.net.URLDecoder;
import java.net.URLEncoder;
import java.security.Key;
import java.security.KeyFactory;
import java.security.KeyStore;
import java.security.PrivateKey;
import java.security.PublicKey;
import java.security.cert.Certificate;
import java.security.spec.PKCS8EncodedKeySpec;
import java.security.spec.X509EncodedKeySpec;

import javax.crypto.Cipher;

import jp.co.sds_corp.test.BasicTestCase;
import jp.co.sds_corp.util.log.CmnLogFactory;
import jp.co.sds_corp.util.log.CmnLogger;

public class CipherTest extends BasicTestCase
{
	CmnLogger logger = CmnLogFactory.getInstance().getLogger(this);
	static final String plain = "abc123";

	public void setUp() throws Exception {
		super.setUp();
	}

	public void testPKCS8() throws Exception {
		String path = "target/test-classes/public_key.der";
		File keyFile = new File(path);
		byte[] encodedKey = new byte[(int)keyFile.length()];

		FileInputStream fis = new FileInputStream(keyFile);
		fis.read(encodedKey);
		fis.close();

		KeyFactory keyFactory = KeyFactory.getInstance("RSA");
		X509EncodedKeySpec publicKeySpec = new X509EncodedKeySpec(encodedKey);
		PublicKey publicKey = keyFactory.generatePublic(publicKeySpec);
		logger.debug("publicKey=" + publicKey.getAlgorithm());

		Cipher cipher = Cipher.getInstance("RSA");
		cipher.init(Cipher.ENCRYPT_MODE, publicKey);
		String crypted = new Base64Encoder().encode(cipher.doFinal(plain.getBytes()));
		String crypted2 = URLEncoder.encode(crypted, "UTF-8");
		logger.debug("crypted=" + crypted + " cryptrd2=" + crypted2);


		path = "target/test-classes/private_key.der";
		keyFile = new File(path);
		encodedKey = new byte[(int)keyFile.length()];

		fis = new FileInputStream(keyFile);
		fis.read(encodedKey);
		fis.close();

		PKCS8EncodedKeySpec privateKeySpec = new PKCS8EncodedKeySpec(encodedKey);
		PrivateKey privateKey = keyFactory.generatePrivate(privateKeySpec);
		logger.debug("privateKey=" + privateKey.getAlgorithm() + privateKey);

		cipher = Cipher.getInstance("RSA");
		cipher.init(Cipher.DECRYPT_MODE, privateKey);

		String crypted3 = URLDecoder.decode(crypted2, "UTF-8");
		logger.debug("crypted3=" + crypted3);
		byte[] crypted4 = new Base64Encoder().decode(crypted3);
		byte[] decrypted = cipher.doFinal(crypted4);
		logger.debug("decrypted=" + new String(decrypted));
	}

	public void testKeystore() throws Exception {
		FileInputStream fis = new FileInputStream("target/test-classes/keystore");
		KeyStore store = KeyStore.getInstance("JKS");
		store.load(fis, "xsw23edc".toCharArray());

		Certificate cert = store.getCertificate("sample");
		logger.debug("cert=" + cert);

		PublicKey publicKey = cert.getPublicKey();
		logger.debug("publicKey=" + publicKey.getAlgorithm());
		Cipher cipher = Cipher.getInstance("RSA"); //publicKey.getAlgorithm()); //"AES/CBC/PKCS5Padding");
		cipher.init(Cipher.ENCRYPT_MODE, publicKey);
		String crypted = new Base64Encoder().encode(cipher.doFinal(plain.getBytes()));
		String crypted2 = URLEncoder.encode(crypted, "UTF-8");

		logger.debug("crypted=" + crypted);
		logger.debug("crypted=" + crypted2);

		Key privateKey = store.getKey("sample", "zaq12wsx".toCharArray());
		logger.debug("privateKey=" + privateKey.getAlgorithm() + privateKey);
		cipher = Cipher.getInstance("RSA"); //privateKey.getAlgorithm());
		cipher.init(Cipher.DECRYPT_MODE, privateKey);
		String crypt1 = URLDecoder.decode(crypted2, "UTF-8");
		logger.debug("crypt1=" + crypt1);
		byte[] cryptBuf = new Base64Encoder().decode(crypt1);
		String decrypted = new String(cipher.doFinal(cryptBuf));
		logger.debug("decrypted=" + decrypted);
	}

}
