package jp.co.sds_corp.util.crypt;

import java.io.File;
import java.io.FileInputStream;

import jp.co.sds_corp.test.BasicTestCase;
import jp.co.sds_corp.util.log.CmnLogFactory;
import jp.co.sds_corp.util.log.CmnLogger;

public class PKCS8CipherTest extends BasicTestCase
{
	CmnLogger logger = CmnLogFactory.getInstance().getLogger(this);
	static final String plain = "abc123";

	public void setUp() throws Exception {
		super.setUp();
	}

	public void testEncrypt1() throws Exception {
		logger.debug("===== testEncrypt1");
		ICipher cipher = new PKCS8Cipher(new File("target/test-classes/private_key.der"));
		byte[] encrypted = cipher.encrypt(plain.getBytes());
		logger.debug("encrypted=" + new Base64Encoder().encode(encrypted));

		ICipher decipher = new X509Cipher(new File("target/test-classes/public_key.der"));
		byte[] decrypted = decipher.decrypt(encrypted);
		String sdec = new String(decrypted);
		logger.debug("decrypted=" + sdec);

		assertEquals("decrypted", plain, sdec);
	}

	public void testDecrypt1() throws Exception {
		logger.debug("===== testDecrypt1");
		ICipher cipher = new X509Cipher();
		cipher.setInputStream(new FileInputStream("target/test-classes/public_key.der"));
		byte[] encrypted = cipher.encrypt(plain.getBytes());
		logger.debug("encrypted=" + new Base64Encoder().encode(encrypted));

		ICipher decipher = new PKCS8Cipher(new File("target/test-classes/private_key.der"));
		byte[] decrypted = decipher.decrypt(encrypted);
		String sdec = new String(decrypted);
		logger.debug("decrypted=" + sdec);
		assertEquals("decrypted", plain, sdec);
	}

	public void testEncrypt2() throws Exception {
		logger.debug("===== testEncrypt2");
		ICipher cipher = new PKCS8Cipher(new File("target/test-classes/private_key.der"));
		// byte[] plainBuf = new byte[117];
		byte[] plainBuf = ((PKCS8Cipher )cipher).load(new File("target/test-classes/cipher.xml"));
		byte[] encrypted = cipher.encrypt(plainBuf);
		logger.debug("encrypted=" + new Base64Encoder().encode(encrypted));

		ICipher decipher = new X509Cipher(new File("target/test-classes/public_key.der"));
		byte[] decrypted = decipher.decrypt(encrypted);
		String sdec = new String(decrypted);
		logger.debug("decrypted=" + sdec);

		assertEquals("decrypted", new String(plainBuf), sdec);
	}

	public void testDecrypt2() throws Exception {
		logger.debug("===== testDecrypt2");
		ICipher cipher = new X509Cipher();
		cipher.setInputStream(new FileInputStream("target/test-classes/public_key.der"));
		// byte[] plainBuf = new byte[1024];
		byte[] plainBuf = ((X509Cipher )cipher).load(new File("target/test-classes/cipher.xml"));
		byte[] encrypted = cipher.encrypt(plainBuf);
		logger.debug("encrypted=" + new Base64Encoder().encode(encrypted));

		ICipher decipher = new PKCS8Cipher(new File("target/test-classes/private_key.der"));
		byte[] decrypted = decipher.decrypt(encrypted);
		String sdec = new String(decrypted);
		logger.debug("decrypted=" + sdec);
		assertEquals("decrypted", new String(plainBuf), sdec);
	}

}
